@@ -2,7 +2,7 @@
  * Copyright (C) 2003-2008 Warren Dukes <warren.dukes@gmail.com>,
 				Eric Wong <normalperson@yhbt.net>,
 				Daniel Brown <danb@cs.utexas.edu>
- * Copyright (C) 2008-2009 Max Kellermann <max@duempel.org>
+ * Copyright (C) 2008-2010 Max Kellermann <max@duempel.org>
  * Project homepage: http://musicpd.org
 
  * This program is free software; you can redistribute it and/or modify
@@ -62,6 +62,10 @@ SIMPLE_CMD(cmd_stop, mpd_run_stop, 1)
 SIMPLE_CMD(cmd_clear, mpd_run_clear, 1)
 SIMPLE_CMD(cmd_shuffle, mpd_run_shuffle, 1)
 
+#if LIBMPDCLIENT_CHECK_VERSION(2,4,0)
+SIMPLE_CMD(cmd_clearerror, mpd_run_clearerror, 1)
+#endif
+
 SIMPLE_ONEARG_CMD(cmd_save, mpd_run_save, 0)
 SIMPLE_ONEARG_CMD(cmd_rm, mpd_run_rm, 0)
 
@@ -87,9 +91,31 @@ int cmd_add (int argc, char ** argv, struct mpd_connection *conn )
 		mpd_send_add(conn, charset_to_utf8(argv[i]));
 	}
 
-	if (!mpd_command_list_end(conn) || !mpd_response_finish(conn))
+	if (!mpd_command_list_end(conn))
 		printErrorAndExit(conn);
 
+	if (!mpd_response_finish(conn)) {
+#if LIBMPDCLIENT_CHECK_VERSION(2,4,0)
+		if (mpd_connection_get_error(conn) == MPD_ERROR_SERVER) {
+			/* check which of the arguments has failed */
+			unsigned location =
+				mpd_connection_get_server_error_location(conn);
+			if (location < (unsigned)argc) {
+				/* we've got a valid location from the
+				   server */
+				const char *message =
+					mpd_connection_get_error_message(conn);
+				message = charset_from_utf8(message);
+				fprintf(stderr, "error adding %s: %s\n",
+					argv[location], message);
+				exit(EXIT_FAILURE);
+			}
+		}
+#endif
+
+		printErrorAndExit(conn);
+	}
+
 	return 0;
 }
 
@@ -102,7 +128,7 @@ cmd_crop(mpd_unused int argc, mpd_unused char **argv, struct mpd_connection *con
 	if (length < 0) {
 
 		mpd_status_free(status);
-		DIE( "You have to have a playlist longer than 1 song in length to crop" );
+		DIE( "A playlist longer than 1 song in length is required to crop.\n" );
 
 	} else if (mpd_status_get_state(status) == MPD_STATE_PLAY ||
 		   mpd_status_get_state(status) == MPD_STATE_PAUSE) {
@@ -132,8 +158,55 @@ cmd_crop(mpd_unused int argc, mpd_unused char **argv, struct mpd_connection *con
 	}
 }
 
+/**
+ * Returns the id of the current song, but only if it is really
+ * playing (playing or paused).
+ */
+static int
+get_active_song(const struct mpd_status *status)
+{
+	return mpd_status_get_state(status) == MPD_STATE_PLAY ||
+		mpd_status_get_state(status) == MPD_STATE_PAUSE
+		? mpd_status_get_song_id(status)
+		: -1;
+}
+
+/**
+ * Wait until the song changes or until playback is started/stopped.
+ */
+static void
+wait_current(struct mpd_connection *c)
+{
+	if (mpd_connection_cmp_server_version(c, 0, 14, 0) < 0)
+		fprintf(stderr, "warning: MPD 0.14 required for this command\n");
+
+	struct mpd_status *status = mpd_run_status(c);
+	if (status == NULL)
+		printErrorAndExit(c);
+
+	const int old_song = get_active_song(status);
+	mpd_status_free(status);
+
+	int new_song;
+	do {
+		enum mpd_idle idle = mpd_run_idle_mask(c, MPD_IDLE_PLAYER);
+		if (idle == 0)
+			printErrorAndExit(c);
+
+		status = mpd_run_status(c);
+		if (status == NULL)
+			printErrorAndExit(c);
+
+		new_song = get_active_song(status);
+		mpd_status_free(status);
+	} while (new_song == old_song);
+}
+
 int cmd_current(mpd_unused int argc, mpd_unused char ** argv, struct mpd_connection *conn)
 {
+	if (options.wait)
+		wait_current(conn);
+
 	struct mpd_status *status;
 
 	if (!mpd_command_list_begin(conn, true) ||
@@ -524,10 +597,17 @@ int cmd_listall ( int argc, char ** argv, struct mpd_connection *conn )
 		listall = charset_to_utf8(argv[i]);
 
 	do {
-		if (!mpd_send_list_all(conn, listall))
-			printErrorAndExit(conn);
+		if (options.custom_format) {
+			if (!mpd_send_list_all_meta(conn, listall))
+				printErrorAndExit(conn);
 
-		print_filenames(conn);
+			print_entity_list(conn);
+		} else {
+			if (!mpd_send_list_all(conn, listall))
+				printErrorAndExit(conn);
+
+			print_filenames(conn);
+		}
 
 		my_finishCommand(conn);
 
@@ -596,7 +676,6 @@ int cmd_update ( int argc, char ** argv, struct mpd_connection *conn)
 
 int cmd_ls ( int argc, char ** argv, struct mpd_connection *conn )
 {
-	struct mpd_entity *entity;
 	const char *ls;
 	int i = 0;
 
@@ -609,31 +688,7 @@ int cmd_ls ( int argc, char ** argv, struct mpd_connection *conn )
 		if (!mpd_send_list_meta(conn, ls))
 			printErrorAndExit(conn);
 
-		while ((entity = mpd_recv_entity(conn)) != NULL) {
-			const struct mpd_directory *dir;
-			const struct mpd_song *song;
-
-			switch (mpd_entity_get_type(entity)) {
-			case MPD_ENTITY_TYPE_UNKNOWN:
-				break;
-
-			case MPD_ENTITY_TYPE_DIRECTORY:
-				dir = mpd_entity_get_directory(entity);
-				printf("%s\n", charset_from_utf8(mpd_directory_get_path(dir)));
-				break;
-
-			case MPD_ENTITY_TYPE_SONG:
-				song = mpd_entity_get_song(entity);
-				printf("%s\n", charset_from_utf8(mpd_song_get_uri(song)));
-				break;
-
-			case MPD_ENTITY_TYPE_PLAYLIST:
-				break;
-			}
-
-			mpd_entity_free(entity);
-		}
-
+		print_entity_list(conn);
 		my_finishCommand(conn);
 
 	} while (++i < argc && (ls = charset_to_utf8(argv[i])) != NULL);
@@ -643,7 +698,6 @@ int cmd_ls ( int argc, char ** argv, struct mpd_connection *conn )
 
 int cmd_lsplaylists ( int argc, char ** argv, struct mpd_connection *conn )
 {
-	struct mpd_entity *entity;
 	const char * ls = "";
 	int i = 0;
 
@@ -653,16 +707,7 @@ int cmd_lsplaylists ( int argc, char ** argv, struct mpd_connection *conn )
 		if (!mpd_send_list_meta(conn, ls))
 			printErrorAndExit(conn);
 
-		while ((entity = mpd_recv_entity(conn)) != NULL) {
-			if (mpd_entity_get_type(entity) == MPD_ENTITY_TYPE_PLAYLIST) {
-				const struct mpd_playlist *playlist =
-					mpd_entity_get_playlist(entity);
-
-				printf("%s\n", charset_from_utf8(mpd_playlist_get_path(playlist)));
-			}
-			mpd_entity_free(entity);
-		}
-
+		print_entity_list(conn);
 		my_finishCommand(conn);
 
 	} while (++i < argc && (ls = charset_to_utf8(argv[i])) != NULL);
@@ -727,8 +772,11 @@ int cmd_volume ( int argc, char ** argv, struct mpd_connection *conn )
 	} else {
 		status = getStatus(conn);
 
-		printf("volume:%3u%c   \n",
-		       mpd_status_get_volume(status), '%');
+		if (mpd_status_get_volume(status) >= 0)
+			printf("volume:%3i%c   ",
+			       mpd_status_get_volume(status), '%');
+		else
+			printf("volume: n/a   ");
 
 		mpd_status_free(status);
 
@@ -876,6 +924,53 @@ int cmd_crossfade ( int argc, char ** argv, struct mpd_connection *conn )
 	return 0;
 }
 
+#if LIBMPDCLIENT_CHECK_VERSION(2,2,0)
+int cmd_mixrampdb ( int argc, char ** argv, struct mpd_connection *conn )
+{
+	float db;
+
+	if(argc==1) {
+		if(!parse_float(argv[0], &db))
+			DIE("\"%s\" is not a floating point number\n",argv[0]);
+
+		mpd_run_mixrampdb(conn, db);
+                my_finishCommand(conn);
+	}
+	else {
+		struct mpd_status *status;
+		status = getStatus(conn);
+
+		printf("mixrampdb: %f\n", mpd_status_get_mixrampdb(status));
+
+		mpd_status_free(status);
+	}
+	return 0;
+}
+
+int cmd_mixrampdelay ( int argc, char ** argv, struct mpd_connection *conn )
+{
+	float seconds;
+
+	if(argc==1) {
+		if(!parse_float(argv[0], &seconds))
+			DIE("\"%s\" is not a floating point number\n",argv[0]);
+
+		mpd_run_mixrampdelay(conn, seconds);
+		my_finishCommand(conn);
+	}
+	else {
+		struct mpd_status *status;
+		status = getStatus(conn);
+
+		printf("mixrampdelay: %f\n",
+		       mpd_status_get_mixrampdelay(status));
+
+		mpd_status_free(status);
+	}
+	return 0;
+}
+#endif
+
 int
 cmd_version(mpd_unused int argc, mpd_unused char **argv, struct mpd_connection *conn)
 {
@@ -940,10 +1035,24 @@ int cmd_lstab ( int argc, char ** argv, struct mpd_connection *conn )
 int cmd_tab ( int argc, char ** argv, struct mpd_connection *conn )
 {
 	struct mpd_song *song;
+	char empty[] = "";
+	char *dir = empty;
+	char *tmp = NULL;
 
-	if (!mpd_send_list_all(conn, ""))
+	if (argc == 1) {
+		if (strrchr(argv[0], '/')) {
+			dir = strdup(argv[0]);
+			if (!dir) return 0;
+			tmp = strrchr(dir, '/');
+			if (tmp) *tmp = '\0'; // XXX: It's unpossible for tmp to be NULL.
+		}
+	}
+
+	if (!mpd_send_list_all(conn, dir))
 		printErrorAndExit(conn);
 
+	if (*dir) free(dir);
+
 	while ((song = mpd_recv_song(conn)) != NULL) {
 		if (argc != 1 ||
 		    strncmp(mpd_song_get_uri(song), argv[0],
