@@ -2,7 +2,7 @@
  * Copyright (C) 2003-2008 Warren Dukes <warren.dukes@gmail.com>,
 				Eric Wong <normalperson@yhbt.net>,
 				Daniel Brown <danb@cs.utexas.edu>
- * Copyright (C) 2008-2009 Max Kellermann <max@duempel.org>
+ * Copyright (C) 2008-2010 Max Kellermann <max@duempel.org>
  * Project homepage: http://musicpd.org
 
  * This program is free software; you can redistribute it and/or modify
@@ -38,6 +38,7 @@ int cmd_stop(int argc, char **argv, struct mpd_connection *conn);
 int cmd_seek(int argc, char **argv, struct mpd_connection *conn);
 int cmd_clear(int argc, char **argv, struct mpd_connection *conn);
 int cmd_shuffle(int argc, char **argv, struct mpd_connection *conn);
+int cmd_clearerror(int argc, char **argv, struct mpd_connection *conn);
 int cmd_move(int argc, char **argv, struct mpd_connection *conn);
 int cmd_playlist(int argc, char **argv, struct mpd_connection *conn);
 int cmd_listall(int argc, char **argv, struct mpd_connection *conn);
@@ -53,6 +54,8 @@ int cmd_random(int argc, char **argv, struct mpd_connection *conn);
 int cmd_single(int argc, char **argv, struct mpd_connection *conn);
 int cmd_consume(int argc, char **argv, struct mpd_connection *conn);
 int cmd_crossfade(int argc, char **argv, struct mpd_connection *conn);
+int cmd_mixrampdb(int argc, char **argv, struct mpd_connection *conn);
+int cmd_mixrampdelay(int argc, char **argv, struct mpd_connection *conn);
 int cmd_enable( int argc, char **argv, struct mpd_connection *conn);
 int cmd_disable( int argc, char **argv, struct mpd_connection *conn);
 int cmd_outputs(int argc, char **argv, struct mpd_connection *conn);
