@@ -2,7 +2,7 @@
  * Copyright (C) 2003-2008 Warren Dukes <warren.dukes@gmail.com>,
 				Eric Wong <normalperson@yhbt.net>,
 				Daniel Brown <danb@cs.utexas.edu>
- * Copyright (C) 2008-2009 Max Kellermann <max@duempel.org>
+ * Copyright (C) 2008-2010 Max Kellermann <max@duempel.org>
  * Project homepage: http://musicpd.org
 
  * This program is free software; you can redistribute it and/or modify
@@ -94,11 +94,16 @@ static struct command {
 	 "Find songs and add them to the current playlist"},
 	{"list",        1,   -1,  0,    cmd_list,        "<type> [<type> <query>]", "Show all tags of <type>"},
 	{"crossfade",   0,   1,   0,    cmd_crossfade,   "[<seconds>]", "Set and display crossfade settings"},
+#if LIBMPDCLIENT_CHECK_VERSION(2,4,0)
+	{"clearerror",  0,   0,   0,    cmd_clearerror,  "", "Clear the current error"},
+#endif
+#if LIBMPDCLIENT_CHECK_VERSION(2,2,0)
+	{"mixrampdb",   0,   1,   0,    cmd_mixrampdb,   "[<dB>]", "Set and display mixrampdb settings"},
+	{"mixrampdelay",0,   1,   0,    cmd_mixrampdelay,"[<seconds>]", "Set and display mixrampdelay settings"},
+#endif
 	{"update",      0,   -1,  2,    cmd_update,      "[<path>]", "Scan music directory for updates"},
-#if defined(LIBMPDCLIENT_CHECK_VERSION)
 #if LIBMPDCLIENT_CHECK_VERSION(2,1,0)
 	{"sticker",     1,   -1,  0,    cmd_sticker,     "<uri> <get|set|list|del> <args..>", "Sticker management"},
-#endif
 #endif
 	{"stats",       0,   -1,  0,    cmd_stats,       "", "Display statistics about MPD"},
 	{"version",     0,   0,   0,    cmd_version,     "", "Report version of MPD"},
@@ -112,7 +117,7 @@ static struct command {
 	  "Idle until an event occurs" },
 	{ "idleloop", 0, -1, 0, cmd_idleloop, "[events]",
 	  "Continuously idle until an event occurs" },
-	{ "replaygain", 0, -1, 0, cmd_replaygain, "[off|track|ablum]",
+	{ "replaygain", 0, -1, 0, cmd_replaygain, "[off|track|album]",
 	  "Set or display the replay gain mode" },
 
 	/* don't remove this, when mpc_table[i].command is NULL it will terminate the loop */
