@@ -2,7 +2,7 @@
  * Copyright (C) 2003-2008 Warren Dukes <warren.dukes@gmail.com>,
 				Eric Wong <normalperson@yhbt.net>,
 				Daniel Brown <danb@cs.utexas.edu>
- * Copyright (C) 2008-2009 Max Kellermann <max@duempel.org>
+ * Copyright (C) 2008-2010 Max Kellermann <max@duempel.org>
  * Project homepage: http://musicpd.org
  
  * This program is free software; you can redistribute it and/or modify
@@ -29,6 +29,7 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <strings.h>
 #include <stdlib.h>
 #include <ctype.h>
 #include <assert.h>
@@ -134,6 +135,22 @@ int parse_int(const char * str, int * ret)
         return 1; /* success */
 }
 
+/* note - return value is success; the parsed float itself is in ret */
+
+int parse_float(const char * str, float * ret)
+{
+	char * test;
+	float temp;
+
+	temp = strtof(str, &test);
+
+	if(*test != '\0')
+		return 0; /* failure */
+
+	*ret = temp;
+	return 1; /* success */
+}
+
 /* note - simply strips number out of formatting; does not -1 or +1 or change
  * the number in any other way for that matter */
 int parse_songnum(const char * str, int * ret)
@@ -265,7 +282,7 @@ song_value(const struct mpd_song *song, const char *name)
 
 /* this is a little ugly... */
 static char *
-songToFormatedString(struct mpd_song *song,
+songToFormatedString(const struct mpd_song *song,
 		     const char *format, const char ** last)
 {
 	char * ret = NULL;
@@ -402,7 +419,8 @@ songToFormatedString(struct mpd_song *song,
 	return ret;
 }
 
-static void print_formatted_song(struct mpd_song *song, const char * format)
+static void
+print_formatted_song(const struct mpd_song *song, const char * format)
 {
 	char * str = songToFormatedString(song, format, NULL);
 
@@ -412,11 +430,49 @@ static void print_formatted_song(struct mpd_song *song, const char * format)
 	}
 }
 
-void pretty_print_song(struct mpd_song *song)
+void
+pretty_print_song(const struct mpd_song *song)
 {
 	print_formatted_song(song, options.format);
 }
 
+void
+print_entity_list(struct mpd_connection *c)
+{
+	struct mpd_entity *entity;
+	while ((entity = mpd_recv_entity(c)) != NULL) {
+		const struct mpd_directory *dir;
+		const struct mpd_song *song;
+		const struct mpd_playlist *playlist;
+
+		switch (mpd_entity_get_type(entity)) {
+		case MPD_ENTITY_TYPE_UNKNOWN:
+			break;
+
+		case MPD_ENTITY_TYPE_DIRECTORY:
+			dir = mpd_entity_get_directory(entity);
+			printf("%s\n", charset_from_utf8(mpd_directory_get_path(dir)));
+			break;
+
+		case MPD_ENTITY_TYPE_SONG:
+			song = mpd_entity_get_song(entity);
+			if (options.custom_format) {
+				pretty_print_song(song);
+				puts("");
+			} else
+				printf("%s\n", charset_from_utf8(mpd_song_get_uri(song)));
+			break;
+
+		case MPD_ENTITY_TYPE_PLAYLIST:
+			playlist = mpd_entity_get_playlist(entity);
+			printf("%s\n", charset_from_utf8(mpd_playlist_get_path(playlist)));
+			break;
+		}
+
+		mpd_entity_free(entity);
+	}
+}
+
 void
 print_filenames(struct mpd_connection *conn)
 {
