@@ -2,7 +2,7 @@
  * Copyright (C) 2003-2008 Warren Dukes <warren.dukes@gmail.com>,
 				Eric Wong <normalperson@yhbt.net>,
 				Daniel Brown <danb@cs.utexas.edu>
- * Copyright (C) 2008-2009 Max Kellermann <max@duempel.org>
+ * Copyright (C) 2008-2010 Max Kellermann <max@duempel.org>
  * Project homepage: http://musicpd.org
  
  * This program is free software; you can redistribute it and/or modify
@@ -42,9 +42,15 @@ int stdinToArgArray(char *** array);
 int get_boolean (const char * arg);
 
 int parse_int(const char *, int *);
+int parse_float(const char *, float *);
 int parse_songnum(const char *, int *);
 int parse_int_value_change(const char *, struct int_value_change *);
-void pretty_print_song(struct mpd_song *song);
+
+void
+pretty_print_song(const struct mpd_song *song);
+
+void
+print_entity_list(struct mpd_connection *c);
 
 void
 print_filenames(struct mpd_connection *conn);
